@@ -3,13 +3,12 @@ Utilities for reading data files.
 """
 
 from . import datafile, location, dict_entry, parse_csv, loadfile_utils
-from ..utils import funcargparse
-from ..utils.library_parameters import library_parameters
+from ..utils import funcargparse, library_parameters
 
 import numpy as np
 
-library_parameters.update({"fileio/loadfile/csv/out_type":"pandas"})
-library_parameters.update({"fileio/loadfile/dict/inline_out_type":"pandas"})
+library_parameters.library_parameters.update({"fileio/loadfile/csv/out_type":"pandas"})
+library_parameters.library_parameters.update({"fileio/loadfile/dict/inline_out_type":"pandas"})
 
 
 ##### File formats #####
@@ -75,7 +74,7 @@ class CSVTableInputFileFormat(ITextInputFileFormat):
     """
     def __init__(self, out_type="default", dtype="numeric", columns=None, delimiters=None, empty_entry_substitute=None, ignore_corrupted_lines=True, skip_lines=0):
         ITextInputFileFormat.__init__(self)
-        self.out_type=library_parameters["fileio/loadfile/csv/out_type"] if out_type=="default" else out_type
+        self.out_type=library_parameters.library_parameters["fileio/loadfile/csv/out_type"] if out_type=="default" else out_type
         self.dtype=dtype
         self.columns=columns
         self.delimiters=delimiters or parse_csv._table_delimiters
@@ -123,7 +122,7 @@ class DictionaryInputFileFormat(ITextInputFileFormat):
         ITextInputFileFormat.__init__(self)
         self.case_normalization=case_normalization
         self.inline_dtype=inline_dtype
-        self.inline_out_type=library_parameters["fileio/loadfile/dict/inline_out_type"] if inline_out_type=="default" else inline_out_type
+        self.inline_out_type=library_parameters.library_parameters["fileio/loadfile/dict/inline_out_type"] if inline_out_type=="default" else inline_out_type
         if not entry_format in {"branch","dict_entry","value"}:
             raise ValueError("unrecognized entry format: {0}".format(entry_format))
         self.entry_format=entry_format
@@ -178,7 +177,7 @@ class BinaryTableInputFileFormatter(IInputFileFormat):
     """
     def __init__(self, out_type="default", dtype="<f8", columns=None, packing="flatten", preamble=None, skip_bytes=0):
         IInputFileFormat.__init__(self)
-        self.out_type=library_parameters["fileio/loadfile/csv/out_type"] if out_type=="default" else out_type
+        self.out_type=library_parameters.library_parameters["fileio/loadfile/csv/out_type"] if out_type=="default" else out_type
         self.preamble=preamble or {}
         self.dtype=self.preamble.get("dtype",dtype)
         try:
@@ -253,7 +252,7 @@ def load_csv(path=None, out_type="default", dtype="numeric", columns=None, delim
     Load data table from a CSV/table file.
 
     Args:
-        path (str): path to the file
+        path (str): path to the file of a file-like object
         out_type (str): type of the result: ``'array'`` for numpy array, ``'pandas'`` for pandas DataFrame,
             or ``'default'`` (determined by the library default; ``'pandas'`` by default)
         dtype: dtype of entries; can be either a single type, or a list of types (one per column).
@@ -283,7 +282,7 @@ def load_csv_desc(path=None, loc="file", return_file=False):
     Analogous to :func:`load_dict`, but doesn't allow any additional parameters (which don't matter in this case).
 
     Args:
-        path (str): path to the file
+        path (str): path to the file of a file-like object
         loc (str): location type (``"file"`` means the usual file location; see :func:`.location.get_location` for details)
         return_file (bool): if ``True``, return :class:`.DataFile` object (contains some metainfo); otherwise, return just the file data
     """
@@ -294,7 +293,7 @@ def load_bin(path=None, out_type="default", dtype="<f8", columns=None, packing="
     Load data from the binary file.
 
     Args:
-        path (str): path to the file
+        path (str): path to the file of a file-like object
         out_type (str): type of the result: ``'array'`` for numpy array, ``'pandas'`` for pandas DataFrame,
             or ``'default'`` (determined by the library default; ``'pandas'`` by default)
         dtype: :class:`numpy.dtype` describing the data.
@@ -321,7 +320,7 @@ def load_bin_desc(path=None, loc="file", return_file=False):
     Analogous to :func:`load_dict`, but doesn't allow any additional parameters (which don't matter in this case).
 
     Args:
-        path (str): path to the file
+        path (str): path to the file of a file-like object
         loc (str): location type (``"file"`` means the usual file location; see :func:`.location.get_location` for details)
         return_file (bool): if ``True``, return :class:`.DataFile` object (contains some metainfo); otherwise, return just the file data
     """
@@ -332,7 +331,7 @@ def load_dict(path=None, case_normalization=None, inline_dtype="generic", entry_
     Load data from the dictionary file.
 
     Args:
-        path (str): path to the file
+        path (str): path to the file of a file-like object
         case_normalization (str): If ``None``, the dictionary paths are case-sensitive;
             otherwise, defines the way the entries are normalized (``'lower'`` or ``'upper'``).
         inline_dtype (str): dtype for inlined tables.
@@ -365,7 +364,7 @@ def load_generic(path=None, file_format=None, loc="file", return_file=False, **k
     Load data from the file.
     
     Args:
-        path (str): path to the file
+        path (str): path to the file of a file-like object
         file_format (str): input file format; if ``None``, attempt to auto-detect file format (same as ``'generic'``);
             can also be an :class:`IInputFileFormat` instance for specific reading method
         loc (str): location type (``"file"`` means the usual file location; see :func:`.location.get_location` for details)
