@@ -7,6 +7,8 @@ import threading
 import os, signal
 import functools
 import collections
+import contextlib
+import cProfile
 from . import functions
 
 
@@ -122,12 +124,12 @@ def recursive_map(value, func):
     return func(value)
 def make_flat_namedtuple(nt, fields=None, name=None, subfield_fmt="{field:}_{subfield:}"):
     """
-    Turn a nested structure of named tuples into a single flat nameduple.
+    Turn a nested structure of named tuples into a single flat namedtuple.
 
     Args:
         nt: toplevel namedtuple class to be flattened
         fields: a dictionary ``{name: desc}`` of the fields, where ``name`` is the named tuple name,
-            and ``desc`` is either a nested nameduple class, or a list of arguments which are passed to the
+            and ``desc`` is either a nested namedtuple class, or a list of arguments which are passed to the
             recursive call to this function (e.g., ``[TTuple, {"field": TNestedTuple}]``).
             Any tuple field which is present in this dictionary gets recursively flattened,
             and the field names of the corresponding returned tuple are added to the full list of fields
@@ -540,6 +542,10 @@ class UIDGenerator:
             self._lock=threading.Lock()
         else:
             self._lock=DummyResource()
+    def reset(self, value=0):
+        """Reset the generator to the given value"""
+        with self._lock:
+            self._value=value
     def __call__(self, inc=True):
         """
         Return a new unique numeric ID.
@@ -547,9 +553,10 @@ class UIDGenerator:
         If ``inc==False``, don't increase the internal counter (the next call will return the same ID). 
         """
         with self._lock:
+            value=self._value
             if inc:
                 self._value=self._value+1
-            return self._value
+            return value
             
 class NamedUIDGenerator:
     """
@@ -632,35 +639,73 @@ class Countdown:
     
     Args:
         timeout (float): Countdown timeout; if ``None``, assumed to be infinite.
+        start (bool): if ``True``, automatically start the countdown; otherwise, wait until :meth:`trigger` is called explicitly
     """
-    def __init__(self, timeout):
+    def __init__(self, timeout, start=True):
         self.timeout=timeout
-        self.reset()
-    def reset(self):
+        self.reset(start=start)
+    def reset(self, start=True):
         """Restart the countdown from the current moment"""
-        self.start=time.time()
-        if self.timeout is None:
-            self.end=None
-        else:
-            self.end=self.timeout+self.start
-    def time_left(self, bound_below=True):
+        self.start=time.time() if start else None
+        self.set_timeout(self.timeout)
+    def trigger(self, restart=True):
+        """
+        Trigger the countdown.
+
+        If ``restart==True``, restart the countdown if it's running; otherwise, do nothing in that situation.
+        """
+        if restart or self.start is None:
+            self.reset()
+    def running(self):
+        """Check if the countdown is running"""
+        return self.start is not None
+    def stop(self):
+        """Stop the timer if currently running"""
+        self.reset(start=False)
+    def time_left(self, t=None, bound_below=True):
         """
         Return the amount of time left. For infinite timeout, return ``None``.
         
         If ``bound_below==True``, instead of negative time return zero.
+        If `t` is supplied, it indicates the current time; otherwise, use ``time.time()``.
         """
+        if self.start is None:
+            return None
         if self.timeout==0 or self.timeout is None:
             return self.timeout
-        dtime=self.end-time.time()
+        t=t or time.time()
+        dtime=self.end-t
         if bound_below:
             dtime=max(dtime,0.)
         return dtime
+    def add_time(self, dt, t=None, bound_below=True):
+        """
+        Add a given amount of time (positive or negative) to the start time (timeout stays the same).
+
+        If ``bound_below==True``, do not let the end time (start time plus timeout) to get below the current time.
+        If `t` is supplied, it indicates the current time; otherwise, use ``time.time()``.
+        """
+        if self.start is None:
+            return
+        self.start+=dt
+        if self.end is not None:
+            self.end+=dt
+            if bound_below:
+                t=t or time.time()
+                self.end=max(self.end,t)
+    def set_timeout(self, timeout):
+        """Change the timer timeout"""
+        self.timeout=timeout
+        if self.timeout is None or self.start is None:
+            self.end=None
+        else:
+            self.end=self.timeout+self.start
     def time_passed(self):
-        """Return the amount of time passed since the countdown start/reset"""
-        return time.time()-self.start
+        """Return the amount of time passed since the countdown start/reset, or ``None`` if it is not started"""
+        return None if self.start is None else time.time()-self.start
     def passed(self):
         """Check if the timeout has passed"""
-        if self.timeout is None:
+        if self.timeout is None or self.start is None:
             return False
         elif self.timeout==0:
             return True
@@ -826,7 +871,35 @@ def setbp():
         import pdb
         return pdb.set_trace
 
-
+@contextlib.contextmanager
+def timing(n=1, name=None, profile=False):
+    """
+    Context manager for timing a piece of code.
+    
+    Measures the time it takes to execute the wrapped code and prints the result.
+    
+    Args:
+        n: can specify the number of repetitions, which is used to show time per single repetition.
+        name: name which is printed alongside the time
+        profile: if ``True``, use ``cProfile`` and print its output instead of a simple timing
+    """
+    if profile:
+        p=cProfile.Profile()
+        p.enable()
+    n=max(n,1)
+    t0=time.time()
+    yield n
+    dt=(time.time()-t0)/n
+    if dt>1:
+        ts="{:.4}s".format(dt)
+    elif dt>1E-3:
+        ts="{:.4}ms".format(dt*1E3)
+    else:
+        ts="{:.4}us".format(dt*1E6)
+    print("{}: {}".format(name or "none",ts))
+    if profile:
+        p.disable()
+        p.print_stats(sort=2)
 
 ### Iterators ###
 
