@@ -1,7 +1,7 @@
 # pylint: disable=wrong-spelling-in-comment
 
 from ...core.utils import ctypes_wrap, files
-from ...core.devio import DeviceError
+from ...core.devio.comm_backend import DeviceError
 from .wlmData_defs import EGetError, ESetError, drEGetError, drESetError  # pylint: disable=unused-import
 from .wlmData_defs import EInst, ECtrlMode, EBaseOperation, EAddOperation  # pylint: disable=unused-import
 from .wlmData_defs import EMeasUnit, ECalibration, EAutocalibration, EEvent  # pylint: disable=unused-import
@@ -92,7 +92,8 @@ class WlmDataLib:
             elif not isinstance(location,tuple):
                 location=(location,)
             all_locations=location+all_locations
-        self.lib,_,path=load_lib.load_lib("wlmData.dll",locations=all_locations,call_conv="stdcall",return_location=True)
+        error_message="The library is located in the folder with HighFinesse wavemeter software"
+        self.lib,_,path=load_lib.load_lib("wlmData.dll",locations=all_locations,call_conv="stdcall",return_location=True,error_message=error_message)
         self.setup_app_path(path)
 
         lib=self.lib
