@@ -11,7 +11,7 @@ from .NIIMAQdx_defs import IMAQdxVideoMode
 from .NIIMAQdx_defs import define_functions
 
 from ...core.utils import ctypes_wrap, py3
-from ...core.devio import DeviceError
+from ...core.devio.comm_backend import DeviceError
 from ..utils import load_lib
 
 import ctypes
@@ -26,6 +26,7 @@ class IMAQdxLibError(IMAQdxError):
         self.func=func
         self.code=code
         self.name=drIMAQdxError.get(self.code,"UNKNOWN")
+        self.desc=""
         try:
             if lib is not None:
                 self.desc=py3.as_str(lib.IMAQdxGetErrorString(code))
@@ -201,7 +202,7 @@ class IMAQdxLib:
 
     def to_API_string(self, value):
         return ctypes.create_string_buffer(py3.as_builtin_bytes(value),IMAQdxAPIStringLength)
-    numeric_attr_types=[IMAQdxAttributeType.IMAQdxAttributeTypeU32, IMAQdxAttributeType.IMAQdxAttributeTypeI64, IMAQdxAttributeType.IMAQdxAttributeTypeBool]
+    numeric_attr_types=[IMAQdxAttributeType.IMAQdxAttributeTypeU32, IMAQdxAttributeType.IMAQdxAttributeTypeI64, IMAQdxAttributeType.IMAQdxAttributeTypeF64, IMAQdxAttributeType.IMAQdxAttributeTypeBool]
     def _new_attr_value(self, attr_type):
         if attr_type==IMAQdxAttributeType.IMAQdxAttributeTypeU32:
             return ctypes.c_uint32()
