@@ -1,15 +1,17 @@
 from . import pfcam_lib
 from .pfcam_lib import lib, PFCamError, PFCamLibError
 
-from ..IMAQ.IMAQ import IMAQCamera, IMAQError
+from ..IMAQ.IMAQ import IMAQFrameGrabber
+from ..SiliconSoftware.fgrab import SiliconSoftwareFrameGrabber
 from ...core.utils import py3, dictionary
-from ...core.devio import DeviceError
+from ...core.devio.comm_backend import DeviceError
 from ..interface import camera
 from ..utils import load_lib
 
 import numpy as np
 import collections
 import re
+import warnings
 
 
 
@@ -45,7 +47,7 @@ def list_cameras(only_supported=True):
     
     If ``only_supported==True``, only return cameras which support PFCam protocol
     (this check only works if the camera is not currently accessed by some other software).
-    Return a list ``[(port, info)]``, where ``port`` is the pfcam port given to :class:`PhotonFocusIMAQCamera`,
+    Return a list ``[(port, info)]``, where ``port`` is the pfcam port given to :class:`IPhotonFocusCamera` and its subclasses,
     and ``info`` is the information returned by :func:`query_camera_name`.
     """
     libctl.preinit()
@@ -53,11 +55,17 @@ def list_cameras(only_supported=True):
     if only_supported:
         ports=[p for p in ports if query_camera_name(p) is not None]
     infos=[lib.pfPortInfo(p) for p in ports]
-    infos=[TCameraInfo(py3.as_str(manu),py3.as_str(name),version,typ) for manu,name,version,typ in infos]
+    infos=[TCameraInfo(py3.as_str(manu),py3.as_str(port),version,typ) for manu,port,version,typ in infos]
     return list(zip(ports,infos))
 def get_cameras_number(only_supported=True):
     """Get the total number of connected PFCam cameras"""
     return len(list_cameras(only_supported=only_supported))
+def get_port_index(manufacturer, port):
+    """Find PhotonFocus port index based on the manufacturer and port"""
+    cams=list_cameras(only_supported=False)
+    for p,d in cams:
+        if d.manufacturer==manufacturer and d.port==port:
+            return p
 
 
 
@@ -180,26 +188,28 @@ class PFCamAttribute:
 
 
 TDeviceInfo=collections.namedtuple("TDeviceInfo",["model","serial_number","grabber_info"])
-class PhotonFocusIMAQCamera(IMAQCamera, camera.IAttributeCamera):
+class IPhotonFocusCamera(camera.IAttributeCamera): # pylint: disable=abstract-method
     """
-    IMAQ+PFCam interface to a PhotonFocus camera.
+    Generic PFCam interface to a PhotonFocus camera.
+    Does not handle frames acquisition, so needs to be mixed with a frame grabber class to be fully operational.
+    In this mixing, the class attribute ``GrabberClass`` should be set to this frame grabber class.
 
     Args:
-        imaq_name: IMAQ interface name (can be learned by :func:`.IMAQ.list_cameras`; usually, but not always, starts with ``"img"``)
         pfcam_port: port number for pfcam interface (can be learned by :func:`list_cameras`; port number is the first element of the camera data tuple)
+            can also be a tuple ``(manufacturer, port)``, e.g., ``("National Instruments", "port0")``.
+        kwargs: keyword arguments passed to the frame grabber initialization
     """
     Error=DeviceError
-    def __init__(self, imaq_name="img0", pfcam_port=0):
+    GrabberClass=None
+    def __init__(self, pfcam_port=0, **kwargs):
+        if isinstance(pfcam_port,tuple):
+            pfcam_port=get_port_index(*pfcam_port)
         self.pfcam_port=pfcam_port
         self.pfcam_opened=False
         self.ucav=dictionary.ItemAccessor(self.get_attribute_value,self.update_attribute_value)
-        try:
-            super().__init__(imaq_name)
-        except IMAQError:
-            self.close()
-            raise
+        super().__init__(do_open=False,**kwargs)
 
-        self._add_info_variable("pfcam_port",lambda: self.pfcam_port)
+        self._add_status_variable("baudrate",self.get_baudrate)
         self._add_settings_variable("trigger_interleave",self.get_trigger_interleave,self.set_trigger_interleave)
         self._add_settings_variable("cfr",self.is_CFR_enabled,self.enable_CFR)
         self._add_settings_variable("status_line",self.is_status_line_enabled,self.enable_status_line)
@@ -208,9 +218,11 @@ class PhotonFocusIMAQCamera(IMAQCamera, camera.IAttributeCamera):
         self._add_settings_variable("frame_period",self.get_frame_period,self.set_frame_period)
         self._add_status_variable("frame_timings",self.get_frame_timings)
     
+        self.open()
+    
     def setup_max_baudrate(self):
         """Setup the maximal available baudrate"""
-        brs=[115200,57600,38400,19200,9600,4800,2400,1200]
+        brs=[921600,460800,230400,115200,57600,38400,19200,9600,4800,2400,1200]
         try:
             for br in brs:
                 if lib.pfIsBaudRateSupported(self.pfcam_port,br):
@@ -218,24 +230,34 @@ class PhotonFocusIMAQCamera(IMAQCamera, camera.IAttributeCamera):
                     return
         except PFCamLibError: # pfSetBaudRate sometimes raises unknown error
             pass
+    def get_baudrate(self):
+        """Get the current baud rate"""
+        return lib.pfGetBaudRate(self.pfcam_port)
     def open(self):
         """Open connection to the camera"""
-        IMAQCamera.open(self)
-        libctl.preinit()
-        if not self.pfcam_opened:
-            lib.pfDeviceOpen(self.pfcam_port)
-            self.pfcam_opened=True
-            self.setup_max_baudrate()
-            self._update_attributes()
-            self._update_imaq()
-            self._hstep=self._get_roi_step("h")
-            self._vstep=self._get_roi_step("v")
+        super().open()
+        try:
+            libctl.preinit()
+            if not self.pfcam_opened:
+                lib.pfDeviceOpen(self.pfcam_port)
+                self.pfcam_opened=True
+                self.setup_max_baudrate()
+                self._update_attributes()
+                self._update_grabber_roi()
+                self._hstep=self._get_roi_step("h")
+                self._vstep=self._get_roi_step("v")
+        except self.Error:
+            super().close()
+            raise
     def close(self):
         """Close connection to the camera"""
-        IMAQCamera.close(self)
+        super().close()
         if self.pfcam_opened:
             lib.pfDeviceClose(self.pfcam_port)
             self.pfcam_opened=False
+    def _get_connection_parameters(self):
+        grabber_params=(self.GrabberClass._get_connection_parameters(self),) if self.GrabberClass else ()
+        return (self.pfcam_port,)+grabber_params
 
     def _normalize_attribute_name(self, name):
         return name.replace(".","/")
@@ -306,7 +328,7 @@ class PhotonFocusIMAQCamera(IMAQCamera, camera.IAttributeCamera):
         """
         model=py3.as_str(lib.pfProperty_GetName(self.pfcam_port,lib.pfDevice_GetRoot(self.pfcam_port)))
         serial_number=self.get_attribute_value("Header/Serial",default=0)
-        grabber_info=tuple(super().get_device_info())
+        grabber_info=tuple(self.GrabberClass.get_device_info(self)) if self.GrabberClass else None
         return TDeviceInfo(model,serial_number,grabber_info)
 
 
@@ -315,9 +337,10 @@ class PhotonFocusIMAQCamera(IMAQCamera, camera.IAttributeCamera):
         return self.ca["Window/W"].max,self.ca["Window/H"].max # pylint: disable=no-member
     def _get_pf_data_dimensions_rc(self):
         return self.cav["Window/H"],self.cav["Window/W"]
-    def _update_imaq(self):
-        r,c=self._get_pf_data_dimensions_rc()
-        IMAQCamera.set_roi(self,0,c,0,r)
+    def _update_grabber_roi(self):
+        if self.GrabberClass:
+            r,c=self._get_pf_data_dimensions_rc()
+            self.GrabberClass.set_roi(self,0,c,0,r)
     def get_roi(self):
         """
         Get current ROI.
@@ -369,20 +392,23 @@ class PhotonFocusIMAQCamera(IMAQCamera, camera.IAttributeCamera):
             if attr is None or not attr.writable:
                 return
         det_size=self.get_detector_size()
-        imaq_detector_size=IMAQCamera.get_detector_size(self)
+        if self.GrabberClass:
+            grabber_detector_size=self.GrabberClass.get_detector_size(self)
+        else:
+            grabber_detector_size=det_size
         if hend is None:
             hend=det_size[0]
         if vend is None:
             vend=det_size[1]
-        self.ucav["Window/W"]=min(hend-hstart,imaq_detector_size[0])
-        self.ucav["Window/H"]=min(vend-vstart,imaq_detector_size[1])
+        self.ucav["Window/W"]=min(hend-hstart,grabber_detector_size[0])
+        self.ucav["Window/H"]=min(vend-vstart,grabber_detector_size[1])
         self.ucav["Window/X"]=hstart
         self.ucav["Window/Y"]=vstart
-        self.ucav["Window/W"]=min(hend-hstart,imaq_detector_size[0]) # in case the previous assignment truncated
-        self.ucav["Window/H"]=min(vend-vstart,imaq_detector_size[1])
-        self._update_imaq()
+        self.ucav["Window/W"]=min(hend-hstart,grabber_detector_size[0]) # in case the previous assignment truncated
+        self.ucav["Window/H"]=min(vend-vstart,grabber_detector_size[1])
+        self._update_grabber_roi()
         return self.get_roi()
-    def get_roi_limits(self, hbin=1, vbin=1):
+    def get_roi_limits(self, hbin=1, vbin=1): # pylint: disable=unused-argument
         params=[self.ca[p] for p in ["Window/W","Window/H"]]
         minp,maxp=[list(p) for p in zip(*[p.update_limits() for p in params])]
         hlim=camera.TAxisROILimit(minp[0],maxp[0],self._hstep,minp[0],1)
@@ -390,14 +416,26 @@ class PhotonFocusIMAQCamera(IMAQCamera, camera.IAttributeCamera):
         return hlim,vlim
 
     def _get_buffer_bpp(self):
-        bpp=IMAQCamera._get_buffer_bpp(self)
-        attr=self.get_attribute("DataResolution",error_on_missing=False)
-        if attr:
-            res=attr.get_value()
-            m=re.match(r"Res(\d+)Bit",res)
-            if m:
-                bpp=(int(m.group(1))-1)//8+1
+        bpp=self.GrabberClass._get_buffer_bpp(self) if self.GrabberClass else 1
+        ppbpp=self._get_camera_bytepp()
+        if ppbpp is not None:
+            bpp=(ppbpp-1)//8+1
         return bpp
+    def _get_camera_bytepp(self):
+        fmt=self.get_attribute_value("DataResolution",error_on_missing=False)
+        if fmt is None:
+            return None
+        m=re.match(r"(?:res)?(\d+)bit",fmt.lower())
+        if m:
+            return int(m[1])
+        return None
+    def _set_camera_bytepp(self, bpp):
+        fmt=self.get_attribute_value("DataResolution",error_on_missing=False)
+        if fmt:
+            m=re.match(r"^([^\d]*)(\d+)([^\d]*)$",fmt)
+            if m:
+                fmt="{}{}{}".format(m[1],bpp,m[3])
+                self.cav["DataResolution"]=fmt
     def _get_data_dimensions_rc(self):
         roi=self.get_roi()
         w,h=roi[1]-roi[0],roi[3]-roi[2]
@@ -476,6 +514,94 @@ class PhotonFocusIMAQCamera(IMAQCamera, camera.IAttributeCamera):
 
 
 
+class PhotonFocusIMAQCamera(IPhotonFocusCamera,IMAQFrameGrabber):
+    """
+    IMAQ+PFCam interface to a PhotonFocus camera.
+
+    Args:
+        imaq_name: IMAQ interface name (can be learned by :func:`.IMAQ.list_cameras`; usually, but not always, starts with ``"img"``)
+        pfcam_port: port number for pfcam interface (can be learned by :func:`list_cameras`; port number is the first element of the camera data tuple)
+            can also be a tuple ``(manufacturer, port)``, e.g., ``("National Instruments", "port0")``.
+    """
+    Error=DeviceError
+    GrabberClass=IMAQFrameGrabber
+    def __init__(self, imaq_name="img0", pfcam_port=0):
+        super().__init__(pfcam_port=pfcam_port,name=imaq_name)
+
+    def open(self):
+        super().open()
+        self._ensure_pixel_format()
+
+    def _ensure_pixel_format(self):
+        fgbpp=self.get_grabber_attribute_value("BITSPERPIXEL")
+        ppbpp=self._get_camera_bytepp()
+        if ppbpp is not None and ppbpp!=fgbpp:
+            msg=(   "PhotonFocus pixel format {} does not agree with the frame grabber {} bits per pixel; "
+                    "changing PhotonFocus pixel format accordingly; to use the original format, alter the camera file".format(self.cav["DataResolution"],fgbpp))
+            warnings.warn(msg)
+            self._set_camera_bytepp(fgbpp)
+
+
+
+class PhotonFocusSiSoCamera(IPhotonFocusCamera,SiliconSoftwareFrameGrabber):
+    """
+    IMAQ+PFCam interface to a PhotonFocus camera.
+
+    Args:
+        siso_board: Silicon Software board index, starting from 0; available boards can be learned by :func:`.fgrab.list_boards`
+        siso_applet: Silicon Software applet name, which can be learned by :func:`.fgrab.list_applets`;
+            usually, a simple applet like ``"DualLineGray16"`` or ``"MediumLineGray16`` are most appropriate;
+            can be either an applet name, or a direct path to the applet DLL
+        siso_port: Silicon Software port number, if several ports are supported by the camera and the applet
+        pfcam_port: port number for pfcam interface (can be learned by :func:`list_cameras`; port number is the first element of the camera data tuple)
+            can also be a tuple ``(manufacturer, port)``, e.g., ``("National Instruments", "port0")``.
+    """
+    Error=DeviceError
+    GrabberClass=SiliconSoftwareFrameGrabber
+    def __init__(self, siso_board, siso_applet, siso_port=0, pfcam_port=0):
+        super().__init__(pfcam_port=pfcam_port,siso_board=siso_board,siso_applet=siso_applet,siso_port=siso_port)
+
+    def open(self):
+        super().open()
+        self._ensure_pixel_format()
+
+    def _ensure_pixel_format(self):
+        ppbpp=self._get_camera_bytepp()
+        if ppbpp is not None:
+            self.setup_camlink_pixel_format(ppbpp,2)
+
+
+
+def check_grabber_association(cam):
+    """
+    Check if PhotonFocus camera has correct association between the frame grabber and the PFRemote interface.
+    
+    `cam` should be an opened instance of :class:`PhotonFocusIMAQCamera` or :class:`PhotonFocusSiSoCamera`.
+    Note that this function changes camera parameters such as exposure, frame period, ROI, trigger source, and status line.
+    """
+    try:
+        if hasattr(cam,"gav") and "CAMSTATUS" in cam.gav and not cam.gav["CAMSTATUS"]:
+            return False
+        cam.clear_acquisition()
+        cam.set_exposure(0)
+        cam.set_frame_period(0)
+        try:
+            cam.clear_all_triggers()
+        except AttributeError:
+            pass
+        cam.cav["Trigger/Source"]=0
+        cam.set_roi(0,64,0,64)
+        cam.enable_status_line()
+        img=cam.snap(timeout=1)
+        if get_status_lines(img) is None:
+            return False
+        cam.enable_status_line(False)
+        img=cam.snap(timeout=1)
+        if get_status_lines(img) is not None:
+            return False
+        return True
+    except cam.Error:
+        return False
 
 ##### Dealing with status line #####
 
@@ -516,10 +642,10 @@ def get_status_lines(frames, check_transposed=True, drop_magic=True):
         s=1 if drop_magic else 0
         lines=_extract_line(frames,True)
         if _check_magic(lines):
-            return lines[:,s:]
+            return lines[...,s:]
         lines=_extract_line(frames,False)
         if _check_magic(lines):
-            return lines[:,s:]
+            return lines[...,s:]
     if check_transposed:
         tframes=frames.T if frames.ndim==2 else frames.transpose((0,2,1))
         return get_status_lines(tframes,check_transposed=False,drop_magic=drop_magic)
@@ -559,7 +685,7 @@ def remove_status_line(frame, sl_pos="calculate", policy="duplicate", copy=True)
             ``"median"`` (set it to the image median), or ``"duplicate"`` (set it equal to the previous row; default)
         copy: if ``True``, make copy of the original frames; otherwise, attempt to remove the line in-place
     """
-    if sl_pos is "calculate":
+    if sl_pos=="calculate":
         sl_pos=get_status_line_position(frame) if frame.ndim==2 else get_status_line_position(frame[0])
     if sl_pos and policy!="keep":
         if copy:
