@@ -31,7 +31,7 @@ libctl=LibraryController(lib)
 
 
 def list_cameras():
-    """Get number of connected TLCamera cameras"""
+    """List connected TLCamera cameras"""
     with libctl.temp_open():
         try:
             return py3.as_str(lib.tl_camera_discover_available_cameras()).split()
@@ -61,7 +61,7 @@ class ThorlabsTLCamera(camera.IBinROICamera, camera.IExposureCamera):
     _frameinfo_fields=TFrameInfo._fields
     def __init__(self, serial=None):
         super().__init__()
-        self.serial=serial
+        self.serial=str(serial) if isinstance(serial,int) else serial
         self.handle=None
         self._buffer=self.RingBuffer()
         self._new_frame_cb=None
@@ -70,8 +70,8 @@ class ThorlabsTLCamera(camera.IBinROICamera, camera.IExposureCamera):
         
         self._add_info_variable("device_info",self.get_device_info)
         self._add_settings_variable("trigger_mode",self.get_trigger_mode,self.set_trigger_mode)
-        self._add_settings_variable("ext_trigger",self.get_ext_trigger_parameters,self.setup_ext_trigger)
-        self._add_settings_variable("hotpixel_correction",self.get_pixel_correction_parameters,self.setup_pixel_correction)
+        self._add_settings_variable("ext_trigger",self.get_ext_trigger_parameters,self.setup_ext_trigger,ignore_error=self.Error)
+        self._add_settings_variable("hotpixel_correction",self.get_pixel_correction_parameters,self.setup_pixel_correction,ignore_error=(self.Error,OSError))  # sometimes raises OSError; DLL issues?
         self._add_info_variable("timestamp_clock_frequency",self.get_timestamp_clock_frequency)
         self._add_status_variable("frame_period",self.get_frame_period)
         
@@ -323,7 +323,7 @@ class ThorlabsTLCamera(camera.IBinROICamera, camera.IExposureCamera):
         if hend-hstart==hlim.min and vend-vstart==vlim.min: # seems to not work for the absolute minimal roi
             if vend<vlim.max:
                 vend+=1
-            else:
+            elif vstart>0:
                 vstart-=1
         lib.tl_camera_set_roi(self.handle,hstart,vstart,hend-1,vend-1)
         return self.get_roi()
