@@ -7,7 +7,7 @@ from .uc480_defs import define_functions
 
 from ...core.utils import ctypes_wrap
 from ..utils import load_lib
-from ...core.devio import DeviceError
+from ...core.devio.comm_backend import DeviceError
 
 import ctypes
 import platform
@@ -68,7 +68,7 @@ class uc480Lib:
         thorcam_path=load_lib.get_program_files_folder("Thorlabs/Scientific Imaging/ThorCam")
         error_message="The library is automatically supplied with Thorcam software\n"+load_lib.par_error_message.format("uc480")
         lib_name="uc480.dll" if platform.architecture()[0][:2]=="32" else "uc480_64.dll"
-        self.lib=load_lib.load_lib(lib_name,locations=("parameter/uc480",thorcam_path,"global"),error_message=error_message,call_conv="stdcall")
+        self.lib=load_lib.load_lib(lib_name,locations=("parameter/uc480",thorcam_path,"global"),error_message=error_message,call_conv="cdecl")
         lib=self.lib
         define_functions(lib)
 
